/* Frobby: Software for monomial ideal computations.
   Copyright (C) 2007 Bjarke Hammersholt Roune (www.broune.com)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see http://www.gnu.org/licenses/.
*/
#ifndef POLYNOMIAL_FACADE_GUARD
#define POLYNOMIAL_FACADE_GUARD

#include "Facade.h"

class BigPolynomial;

/** A facade for operations on polynomials.

    @ingroup Facade
 */
class PolynomialFacade : private Facade {
 public:
  PolynomialFacade(bool printActions);

  /** Sort the terms of poly. */
  void sortTerms(BigPolynomial& poly);

  /** Sort the variables of poly. */
  void sortVariables(BigPolynomial& poly);
};

#endif
