/* Frobby: Software for monomial ideal computations.
   Copyright (C) 2009 Bjarke Hammersholt Roune (www.broune.com)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see http://www.gnu.org/licenses/.
*/
#include "stdinc.h"
#include "TestQualifier.h"

#include "TestSuite.h"

bool TestQualifier::visitEnter(TestSuite& suite) {
  if (!suite.getName().empty()) {
    _path += suite.getName();
    _path += '.';
  }
  return true;
}

bool TestQualifier::visitLeave(TestSuite& suite) {
  if (!suite.getName().empty()) {
    // The -1 to remove the dot as well.
    _path.resize(_path.size() - suite.getName().size() - 1);
  }

  return true;
}

const string& TestQualifier::getPath() const {
  return _path;
}
